#!/usr/bin/bash

# 2025.01.30

# Изоляция сетевого пространстка имён:
# - не запускать контейнер с параметром --net=host (5.09)
# - при необходимости доступа по сети следут выполнить проброс порта на конкретном интерфейсе используя параметр -p HOST_IP:HOST_PORT:CONTAINER_PORT (5.13)
# Ограничение памяти: Необходимо всегда запускать контейнер с параметром --memory 256m (5.10)
# Монтирование корневой файловой системы в режиме только для чтения: следует использовать флаг --read-only (5.12)
#    и при необходимости явно указывать каталоги в которые разрешена запись. Также можно использовать хранилища (volume) docker для изоляции дискового пространства
#

# Изоляция пространства PID процесса: не запускать контейнер с параметром --pid=host (5.15)
# Изоляция пространства IPC процесса: не запускать контейнер с параметром --ipc=host (5.16)
# Изоляция устройств хоста и контейнера: не использовать параметр --device (5.17). При необходимости ограничивать права минимально необходимыми.
# Изоляция пользовательских пространств имён: не запускать контейнер с параметром --userns=host (5.30)

#set -e
#set -x

EXECUTABLE_NAME=`echo $1 | sed -e "s/.*\///g"`

ARGS=`ps -C $EXECUTABLE_NAME -o pid= -o command=`
PID=`echo "$ARGS" | awk '{print $1}'`

if [ "$PID" = "" ]; then
	exit 0;
fi

# Не тестируем параметрвы dosker если запущен не с параметром run
DOCKER_ARG1=`echo "$ARGS" | awk '{print $3}'`
if [ "$DOCKER_ARG1" != "run" ]; then
	exit 0;
fi

# Не тестируем параметрвы dosker если запущен с параметром --help
DOCKER_ARG2=`echo "$ARGS" | awk '{print $4}'`
if [ "$DOCKER_ARG2" == "--help" ]; then
	exit 0;
fi

illegal_param_fn()
{
	STR="docker[$PID]: Использован недопустимы параметр '$1'"
	logger --id=$$ -p daemon.err "$STR"
	kill -9 $PID > /dev/null 2>&1
	echo $STR | docker_n_email_notification_tool.sh docker_watchdog "Контроль параметров docker"
	exit 0
}

#
# Определние запрещённых параметров утилиты docker
#
for ARG in $ARGS ; do
	case $ARG in
		--net=host) illegal_param_fn $ARG;; # 6.1.5. Изоляцию сетевых пространств имен (сетевых интерфейсов) хост системы от пространств имен (сетевых интерфейсов) контейнеров.
		--pid=host) illegal_param_fn $ARG;; # 6.1.1. Изоляцию пространств идентификаторов процессов контейнеров между процессами хост системы и процессами контейнера.
		--ipc=host) illegal_param_fn $ARG;; # 6.1.2. Изоляцию пространств имен процессов хост системы и пространства имён контейнеров для межпроцессного взаимодействия контейнеров.


		--device) illegal_param_fn $ARG;; # 6.4.1. Ограничение прав прикладного программного обеспечения, выполняемого внутри контейнера, на использование периферийных устройств, устройств хранения данных и съемных машинных носителей информации (блочных устройств), входящих в состав информационной (автоматизированной) системы.
						  # 6.1.6. Изоляцию пространств имен файловой системы хоста и пространств имен файловых систем контейнеров.
		--userns=host) illegal_param_fn $ARG;; # 6.1.3. Изоляцию пространств имен пользователей и групп пользователей хост системы и пространств имён пользователей и групп контейнеров.
		*);;
	esac
done

#
# Определние отсутствия обязательных параметров утилиты docker
#
req_param_counter=0

for ARG in $ARGS ; do
	case $ARG in
		--memory) # 6.4.2. Ограничение прав прикладного программного обеспечения, выполняемого внутри контейнера, на использование вычислительных ресурсов (оперативной памяти, операций ввода-вывода за период времени) хостовой операционной системы.
			req_param_counter=$((req_param_counter|1));;
#		--cpu-quota) # 6.4.2. Ограничение прав прикладного программного обеспечения, выполняемого внутри контейнера, на использование вычислительных ресурсов (оперативной памяти, операций ввода-вывода за период времени) хостовой операционной системы.
#			req_param_counter=$((req_param_counter+1));;
#		--device-read-iops) # 6.4.2. Ограничение прав прикладного программного обеспечения, выполняемого внутри контейнера, на использование вычислительных ресурсов (оперативной памяти, операций ввода-вывода за период времени) хостовой операционной системы.
#			req_param_counter=$((req_param_counter+1));;
#		--device-write-iops) # 6.4.2. Ограничение прав прикладного программного обеспечения, выполняемого внутри контейнера, на использование вычислительных ресурсов (оперативной памяти, операций ввода-вывода за период времени) хостовой операционной системы.
#			req_param_counter=$((req_param_counter+1));;
		--read-only) # 6.4.3. Монтирование корневой файловой системы хостовой операционной системы в режиме "только для чтения".
			req_param_counter=$((req_param_counter|2));;
		*);;
	esac
done

if [ $((req_param_counter&1)) -eq 0 ]; then
	kill -9 $PID > /dev/null 2>&1
	STR="docker[$PID]: Отсутствует необходимый параметр: --memory"
	logger --id=$$ -p daemon.err "$STR"
	echo $STR | docker_n_email_notification_tool.sh docker_watchdog "Контроль параметров docker"
fi

if [ $((req_param_counter&2)) -eq 0 ]; then
	kill -9 $PID > /dev/null 2>&1
	STR="docker[$PID]: Отсутствует необходимый параметр: --read-only"
	logger --id=$$ -p daemon.err "$STR"
	echo $STR | docker_n_email_notification_tool.sh docker_watchdog "Контроль параметров docker"
fi
