#!/usr/bin/bash

# eugene
# 2025-02-01

# Запуск постоянного мониторинга за каталогом /home
# Проверяется только верхний уровень
# При появлении новых каталогов:
#   1) daemon.json копируется из системного каталога в каталог пользователя
#   2) для всех каталогов в /home в аудит вносится избранная информация в файл /etc/audit/rules.d/docker_users.rules
#   3) служба аудита перезапускается

WATCHED_DIR="/home"

inotifywait -m -e create "$WATCHED_DIR" --format "%f" | while read NEW_ITEM; do
  # чтобы успел отработаться skel
  sleep 15
  # Проверим, каталог ли это
  if [ -d "$WATCHED_DIR/$NEW_ITEM" ]; then
    # это каталог
    DIR_DJ="$WATCHED_DIR""/""$NEW_ITEM""/.config/docker"
    if [ -d "$DIR_DJ" ]; then
      # При создании нового профиля из /etc/skel должен подтянуться каталог ~/.config/docker
      # А то вдруг root просто каталог в /home создал
      # скопируем daemon.json в каталог пользователя
      cp /etc/docker_security/rootless-config/daemon.json "$DIR_DJ"
      FILE_DJ="$DIR_DJ""/""daemon.json"
      # установим права, запрещающие пользователю изменять этот файл
      chown root:root "$FILE_DJ"
      # сделаем файл защищенным
      chattr +i "$FILE_DJ"
      # Внесем в аудит новые данные
      FILE_AUDIT="/etc/audit/rules.d/docker_users.rules"
      cd /home
      # аудит за всем каталогом ../.local/share/docker может генерировать большие объемы данных и вызывать быстрое заполнение дискового пространства
      # в таком случае для аудита можно выбрать более простой режим контроля
      # для этого необходимо закомментировать первую строку find... , следующую за этим комментарием, и расскомментировать следующую строку find...
      find * -maxdepth 0 -type d | xargs -izDIRz echo -e '-w /home/zDIRz/.docker/ -k docker\n-w /home/zDIRz/.config/docker/ -k docker\n-w /home/zDIRz/.config/systemd/ -k docker\n-w /home/zDIRz/.local/share/docker/ -k docker' > "$FILE_AUDIT"
      # find * -maxdepth 0 | xargs -izDIRz echo -e '-w /home/zDIRz/.docker/ -k docker\n-w /home/zDIRz/.config/docker/ -k docker\n-w /home/zDIRz/.config/systemd/ -k docker' > "$FILE_AUDIT"
      service auditd restart
      if [ $? -ne 0 ]; then
        logger --id=$$ "$(basename "$0") Ошибка: при перезапуске сервиса аудита"
      else
        logger --id=$$ "$(basename "$0") Инфо: сервис аудита перезапущен"
      fi
    fi # if [ -d "$DIR_DJ" ]; then
  fi # if [ -d "$WATCHED_DIR/$NEW_ITEM" ]; then
done
