#!/usr/bin/bash

# eugene
# 2025.02.03

# Поиск всех запущенных Docker и запуск сканеров


# ПРЕДВАРИТЕЛЬНАЯ ПРОВЕРКА

# Подключение сервисного файла
FILE_SCRIPTS_FUNCTIONS="/usr/sbin/docker_s_functions"

# Убедитесь, что файл существует
if [ ! -f "$FILE_SCRIPTS_FUNCTIONS" ]; then
  echo "$(basename "$0") Ошибка: Файл не найден: $FILE_SCRIPTS_FUNCTIONS."
  logger --id=$$ "$(basename "$0") Ошибка: Файл не найден: $FILE_SCRIPTS_FUNCTIONS."
  exit 1
fi

# Подключим source
source "$FILE_SCRIPTS_FUNCTIONS" || { \
  echo "$(basename "$0") Ошибка: Не подключен source файл: $FILE_SCRIPTS_FUNCTIONS."
  logger --id=$$ "$(basename "$0") Ошибка: Не подключен source файл: $FILE_SCRIPTS_FUNCTIONS."
  exit 1
}

# Имя файла лога
LOG_FILE_NAME="scanners.log"

# Имя файла лога полное
LOG_FILE_NAME_FULL="$DIR_LOGS"/"$LOG_FILE_NAME"

# Убедитесь, что Docker установлен
func_check_packages "docker-ce-rootless-extras" "$LOG_FILE_NAME_FULL"
if [ $? -ne 0 ]; then
  exit 1
fi


# ФУНКЦИИ СКРИПТА

# ФУНКЦИЯ сканирования конкретного docker.sock
func_bench_scan_docker_user() {
  local username=$1
  local uid=$2
  local docker_socket=$3

  local userUID="$username (UID: $uid)"

  func_echo_info "Сканирование (bench) для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"

  # Установите среду Docker для сокета конкретного пользователя
  export DOCKER_HOST="unix://$docker_socket"

  # Проверьте доступ к Docker
  if ! docker info &>/dev/null; then
    func_echo_error "Пропустим... Не удалось получить доступ к Docker для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"
    return 1
  fi

  func_check_file "$FILE_SCRIPTS_BENCH" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # Создайте переменную с отметкой времени
  local timestamp
  timestamp=$(date +"%Y-%m-%d_%H-%M-%S")

  # Создайте временный каталог со сложным случайным именем
  local temp_dir
  local temp_suffix="bench_scan_temp_XXXXXXXX"
  temp_dir=$(mktemp -d -t ${temp_suffix})
  func_check_mkdir "${temp_dir}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # каталог для отчетов
  local report_dir="${temp_dir}"

  # Создайте каталог для txt файлов
  local report_dir_txt="${report_dir}/txt"
  func_check_mkdir "${report_dir_txt}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  FILE_PATH="${report_dir_txt}/bench_scan_reports_${username}.txt"

  cd "$DIR_SCRIPTS_BENCH"

  "$FILE_SCRIPTS_BENCH" > "$FILE_PATH"

  cd /usr/sbin

  # Будет потом телом письма
  local file_body="$report_dir/BODY_FILE"
  # Будет потом вложением
  local file_attachment="$report_dir/ATTACHMENT_FILE.tar.gz"
  # Будет потом заголовком письма
  local file_subject="$report_dir/SUBJECT_FILE"

  # после сканирования предупреждения WARN были?
  local warn=$(grep "\[WARN\]" "$FILE_PATH" | wc -l)
  if [[  "$warn" -gt 0  ]]; then
    # т.е. предупреждения есть
    func_echo_info "Замечания по настройкам Docker найдены" "$warn" "$LOG_FILE_NAME_FULL"

    # Создайте файл для будущего заголовка письма
    echo "$timestamp (Bench Scan), Пользователь: $username, Имя хоста: $(hostname), Замечаний WARN: $warn" >"$file_subject"

    # почистим вывод
    grep -i "WARN" "$FILE_PATH" > "$FILE_PATH.1"
    grep -v "AppArmor" "$FILE_PATH.1" > "$FILE_PATH.2"
    sed 's/^.\{18\}//g' "$FILE_PATH.2" > "$FILE_PATH.3"
    # Создайте файл для будущего тела письма
    echo "Замечаний WARN по итогам сканирования настроек Docker: $warn" >"$file_body"
    echo " " >> "$file_body"
    cat "$FILE_PATH.3" >> "$file_body"
    rm -f "$FILE_PATH.1" #&>/dev/null
    rm -f "$FILE_PATH.2" #&>/dev/null
    rm -f "$FILE_PATH.3" #&>/dev/null
  else
    # Создайте файл для будущего заголовка письма
    echo "$timestamp (Bench Scan), Пользователь: $username, Имя хоста: $(hostname), Сканирование без замечаний WARN" >"$file_subject"
    # Создайте файл для будущего тела письма
    echo "Замечаний WARN по итогам сканирования настроек Docker нет" >"$file_body"
  fi

  # Заархивируйте все полученные txt файлы
  tar -czf "$file_attachment" -C "$report_dir_txt" .

  # Создайте итоговый архив
  local final_archive_name_only="${timestamp}_bench_scan_${username}_${uid}.tar.gz"
  local final_archive="$report_dir/${final_archive_name_only}"

  tar -czf "$final_archive" -C "$report_dir" SUBJECT_FILE BODY_FILE ATTACHMENT_FILE.tar.gz

  # Переместите итоговый архив из временного каталога в каталог для отправки
  if [[  -f "$final_archive"  ]]; then
    mv "$final_archive" "$DIR_REPORTS_TO_SEND"
    if [[  $? -eq 0  ]]; then
      echo
      # func_echo_info "Перемещен файл: '$final_archive' в каталог отправки" "$DIR_REPORTS_TO_SEND" "$LOG_FILE_NAME_FULL"
    else
      func_echo_error "Не удалось преместить файл: '$final_archive' в каталог отправки" "$DIR_REPORTS_TO_SEND" "$LOG_FILE_NAME_FULL"
    fi
  else
    func_echo_error "Файл не найден" "$final_archive" "$LOG_FILE_NAME_FULL"
  fi

  func_echo_info "Отчет" "${final_archive_name_only}" "$LOG_FILE_NAME_FULL"
  func_echo_info "Завершено (bench) сканирование для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"
  rm -rf "$temp_dir"
} # func_bench_scan_docker_user()


# ФУНКЦИЯ сканирования конкретного docker.sock
func_trivy_scan_docker_images_for_user() {
  local username=$1
  local uid=$2
  local docker_socket=$3

  local userUID="$username (UID: $uid)"

  func_echo_info "Сканирование (trivy) для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"

  # Установите среду Docker для сокета конкретного пользователя
  export DOCKER_HOST="unix://$docker_socket"

  # Проверьте доступ к Docker
  if ! docker info &>/dev/null; then
    func_echo_error "Пропустим... Не удалось получить доступ к Docker для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"
    return 1
  fi

  # Получите список всех изображений Docker для этого пользователя
  local images
  images=$(docker images --format "{{.Repository}}:{{.Tag}} {{.ID}}")
  if [[ -z "$images" ]]; then
    func_echo_error "Не найдены образы Docker для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"
    return 1
  fi

  # Создайте переменную с отметкой времени
  local timestamp
  timestamp=$(date +"%Y-%m-%d_%H-%M-%S")

  # Создайте временный каталог со сложным случайным именем
  local temp_dir
  local temp_suffix="trivy_scan_temp_XXXXXXXX"
  temp_dir=$(mktemp -d -t ${temp_suffix})
  func_check_mkdir "${temp_dir}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # Создайте каталог для отчетов
  local report_dir="${temp_dir}/scan_reports_${username}"
  func_check_mkdir "${report_dir}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # Создайте каталог для json файлов
  local report_dir_json="${report_dir}/json"
  func_check_mkdir "${report_dir_json}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # Инициализация счетчиков
  local total_images=0
  local failed_scans=0
  local images_with_vulnerabilities=0
  # Будет потом телом письма
  local file_body="$report_dir/BODY_FILE"
  # Будет потом вложением
  local file_attachment="$report_dir/ATTACHMENT_FILE.tar.gz"
  # Будет потом заголовком письма
  local file_subject="$report_dir/SUBJECT_FILE"
  # Общие счетчики уязвимостей для текущего сканирования
  local sum_critical=0
  local sum_high=0
  local sum_medium=0
  local sum_low=0
  local sum_unknown=0
  local sum_total=0

  # Отсканируйте каждый образ и сохраните отчет
  while IFS= read -r line; do
    local image_name
    local image_id
    image_name=$(echo "$line" | awk '{print $1}')
    image_id=$(echo "$line" | awk '{print $2}')
    local safe_image_name
    safe_image_name=$(echo "$image_name" | tr '/:' '__')

    if [[ "$image_name" == "<none>:<none>" ]]; then
      func_echo_error "Пропуск безымянного образа с ID" "$image_id" "$LOG_FILE_NAME_FULL"
      continue
    fi

    # Сгенерируйте уникальное имя файла отчета
    local report_file_name_only="${image_id}_${safe_image_name}.json"
    local report_file="$report_dir_json/$report_file_name_only"

    echo "Инфо: Сканирование образа: $image_name (ID: $image_id)..."

    # Запуск Trivy для сканирования образа
    trivy image --quiet --format json --output "$report_file" "$image_name"
    if [[ $? -eq 0 ]]; then
      # echo "Инфо: Отчет сохранен как: $report_file_name_only."

      # Получим итоги
      local critical=$(grep "\"CRITICAL\"" "$report_file" | wc -l)
      local high=$(grep "\"HIGH\"" "$report_file" | wc -l)
      local medium=$(grep "\"MEDIUM\"" "$report_file" | wc -l)
      local low=$(grep "\"LOW\"" "$report_file" | wc -l)
      local unknown=$(grep "\"UNKNOWN\"" "$report_file" | wc -l)
      local total_vulnerabilities=$((  $critical + $high + $medium + $low + $unknown  ))

      # Получим общие счетчики
      sum_critical=$((sum_critical + critical))
      sum_high=$((sum_high + high))
      sum_medium=$((sum_medium + medium))
      sum_low=$((sum_low + low))
      sum_unknown=$((sum_unknown + unknown))
      sum_total=$((sum_total + total_vulnerabilities))
 
      # тоже работает, но если не находит, то в выводе ругается. Переделал
      #local total_vulnerabilities=$(jq '.Results[].Vulnerabilities | length' "$report_file" | awk '{s+=$1} END {print s+0}')

      # Запись общего итога сканирования образа в файл, который потом будет телом письма
      echo "$image_name (ID: $image_id), CVE: $total_vulnerabilities, critical: $critical, high: $high, medium: $medium, low: $low, unknown: $unknown" >>"$file_body"

      # Подсчитайте количество образов с уязвимостями
      if [[ "$total_vulnerabilities" -gt 0 ]]; then
        images_with_vulnerabilities=$((images_with_vulnerabilities + 1))
      fi
    else
      func_echo_error "Пропустим... Не удалось выполнить сканирование" "$image_name" "$LOG_FILE_NAME_FULL"
      rm -f "$report_file"
      failed_scans=$((failed_scans + 1))
    fi

    total_images=$((total_images + 1))
  done <<<"$images"

  sed -i '1i \ ' "$file_body"
  sed -i '1i \  Подробнее по каждому из образов:' "$file_body"
  sed -i '1i \ ' "$file_body"
  sed -i '1i \  UNKNOWN: '$sum_unknown "$file_body"
  sed -i '1i \  LOW: '$sum_low "$file_body"
  sed -i '1i \  MEDIUM: '$sum_medium "$file_body"
  sed -i '1i \  HIGH: '$sum_high "$file_body"
  sed -i '1i \  CRITICAL: '$sum_critical "$file_body"  
  sed -i '1i \ ' "$file_body"
  sed -i '1i из них' "$file_body"
  sed -i '1i \ ' "$file_body"
  sed -i '1i Всего найдено уязвимостей: '$sum_total "$file_body"
  sed -i '1i \ ' "$file_body"
  sed -i '1i Не удалось отсканировать образов: '$failed_scans "$file_body"
  sed -i '1i Всего образов c уязвимостями: '$images_with_vulnerabilities "$file_body"
  sed -i '1i Всего образов: '$total_images "$file_body"

  # Заархивируйте все полученные JSON файлы
  tar -czf "$file_attachment" -C "$report_dir_json" .

  # Создайте файл для будущего заголовка письма
  echo "$timestamp (Trivy Scan), Пользователь: $username, Имя хоста: $(hostname), Всего образов Docker: $total_images, Всего с уязвимостями: $images_with_vulnerabilities" >"$file_subject"

  # Создайте итоговый архив
  local final_archive_name_only="${timestamp}_trivy_scan_${username}_${uid}.tar.gz"
  local final_archive="$report_dir/${final_archive_name_only}"

  tar -czf "$final_archive" -C "$report_dir" SUBJECT_FILE BODY_FILE ATTACHMENT_FILE.tar.gz

  # Переместите итоговый архив из временного каталога в каталог для отправки
  if [[  -f "$final_archive"  ]]; then
    mv "$final_archive" "$DIR_REPORTS_TO_SEND"
    if [[  $? -eq 0  ]]; then
      echo
      # func_echo_info "Перемещен файл: '$final_archive' в каталог отправки" "$DIR_REPORTS_TO_SEND" "$LOG_FILE_NAME_FULL"
    else
      func_echo_error "Не удалось преместить файл: '$final_archive' в каталог отправки" "$DIR_REPORTS_TO_SEND" "$LOG_FILE_NAME_FULL"
    fi
  else
    func_echo_error "Файл не найден" "$final_archive" "$LOG_FILE_NAME_FULL"
  fi

  func_echo_info "Всего отсканировано образов Docker" "$total_images" "$LOG_FILE_NAME_FULL"

  if [[  "$failed_scans" -gt 0  ]]; then
    func_echo_error "Не удалось отсканировать образов Docker" "$failed_scans" "$LOG_FILE_NAME_FULL"
  fi

  func_echo_info "Отчет" "${final_archive_name_only}" "$LOG_FILE_NAME_FULL"
  func_echo_info "Завершено (trivy) сканирование для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"
  rm -rf "$temp_dir"
} # trivy_scan_docker_images_for_user()


func_dockle_scan_docker_images_for_user() {
  local username=$1
  local uid=$2
  local docker_socket=$3

  local userUID="$username (UID: $uid)"

  func_echo_info "Сканирование (dockle) для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"

  # Установите среду Docker для сокета конкретного пользователя
  export DOCKER_HOST="unix://$docker_socket"

  # Проверьте доступ к Docker
  if ! docker info &>/dev/null; then
    func_echo_error "Пропустим... Не удалось получить доступ к Docker для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"
    return 1
  fi

  # Получите список всех изображений Docker для этого пользователя
  local images
  images=$(docker images --format "{{.Repository}}:{{.Tag}} {{.ID}}")
  if [[ -z "$images" ]]; then
    func_echo_error "Не найдены образы Docker для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"
    return 1
  fi

  # Создайте переменную с отметкой времени
  local timestamp
  timestamp=$(date +"%Y-%m-%d_%H-%M-%S")

  # Создайте временный каталог со сложным случайным именем
  local temp_dir
  local temp_suffix="dockle_scan_temp_XXXXXXXX"
  temp_dir=$(mktemp -d -t ${temp_suffix})
  func_check_mkdir "${temp_dir}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # Создайте каталог для отчетов
  local report_dir="${temp_dir}/scan_reports_${username}"
  func_check_mkdir "${report_dir}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # Создайте каталог для json файлов
  local report_dir_json="${report_dir}/json"
  func_check_mkdir "${report_dir_json}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # Инициализация счетчиков
  local total_images=0
  local failed_scans=0
  local images_with_vulnerabilities=0
  # Будет потом телом письма
  local file_body="$report_dir/BODY_FILE"
  # Будет потом вложением
  local file_attachment="$report_dir/ATTACHMENT_FILE.tar.gz"
  # Будет потом заголовком письма
  local file_subject="$report_dir/SUBJECT_FILE"
  # Общие счетчики уязвимостей для текущего сканирования
  local sum_fatal=0
  local sum_warn=0
  local sum_info=0
  local sum_skip=0
  local sum_pass=0
  local sum_total=0

  # Отсканируйте каждый образ и сохраните отчет
  while IFS= read -r line; do
    local image_name
    local image_id
    image_name=$(echo "$line" | awk '{print $1}')
    image_id=$(echo "$line" | awk '{print $2}')
    local safe_image_name
    safe_image_name=$(echo "$image_name" | tr '/:' '__')

    if [[ "$image_name" == "<none>:<none>" ]]; then
      func_echo_error "Пропуск безымянного образа с ID" "$image_id" "$LOG_FILE_NAME_FULL"
      continue
    fi

    # Сгенерируйте уникальное имя файла отчета
    local report_file_name_only="${image_id}_${safe_image_name}.json"
    local report_file="$report_dir_json/$report_file_name_only"

    echo "Инфо: Сканирование образа: $image_name (ID: $image_id)..."

    # Запуск dockle для сканирования образа
    dockle -f json --output "$report_file" "$image_name"

    if [[ $? -eq 0 ]]; then
      # echo "Инфо: Отчет сохранен как: $report_file_name_only."

      # Получим итоги
      local fatal=$(jq -r '.summary.fatal' "$report_file")
      local warn=$(jq -r '.summary.warn' "$report_file")
      local info=$(jq -r '.summary.info' "$report_file")
      local skip=$(jq -r '.summary.skip' "$report_file")
      local pass=$(jq -r '.summary.pass' "$report_file")
      local total_vulnerabilities=$((  $fatal + $warn  ))

      # Получим общие счетчики
      sum_fatal=$((sum_fatal + fatal))
      sum_warn=$((sum_warn + warn))
      sum_info=$((sum_info + info))
      sum_skip=$((sum_skip + skip))
      sum_pass=$((sum_pass + pass))
      sum_total=$((sum_total + total_vulnerabilities))

      # Запись общего итога сканирования образа в файл, который потом будет телом письма
      echo "$image_name (ID: $image_id), fatal: $fatal, warn: $warn, info: $info, skip: $skip, pass: $pass" >>"$file_body"

      # Подсчитайте количество образов с уязвимостями
      if [[ "$total_vulnerabilities" -gt 0 ]]; then
        images_with_vulnerabilities=$((images_with_vulnerabilities + 1))
      fi
    else
      func_echo_error "Пропустим... Не удалось выполнить сканирование" "$image_name" "$LOG_FILE_NAME_FULL"
      rm -f "$report_file"
      failed_scans=$((failed_scans + 1))
    fi

    total_images=$((total_images + 1))
  done <<<"$images"

  sed -i '1i \ ' "$file_body"
  sed -i '1i \  Подробнее по каждому из образов:' "$file_body"
  sed -i '1i \ ' "$file_body"
  sed -i '1i \  всего INFO:  '$sum_info "$file_body"
  sed -i '1i \  всего WARN:  '$sum_warn "$file_body"
  sed -i '1i \  всего FATAL: '$sum_fatal "$file_body"  
#  sed -i '1i \ ' "$file_body"
#  sed -i '1i Всего найдено оценок FATAL + WARN: '$sum_total "$file_body"
  sed -i '1i \ ' "$file_body"
  sed -i '1i Не удалось отсканировать образов: '$failed_scans "$file_body"
  sed -i '1i Всего образов c замечаниями: '$images_with_vulnerabilities "$file_body"
  sed -i '1i Всего образов: '$total_images "$file_body"

  # Заархивируйте все полученные JSON файлы
  tar -czf "$file_attachment" -C "$report_dir_json" .

  # Создайте файл для будущего заголовка письма
  echo "$timestamp (dockle Scan), Пользователь: $username, Имя хоста: $(hostname), Всего образов Docker: $total_images, Всего с замечаниями: $images_with_vulnerabilities" >"$file_subject"

  # Создайте итоговый архив
  local final_archive_name_only="${timestamp}_dockle_scan_${username}_${uid}.tar.gz"
  local final_archive="$report_dir/${final_archive_name_only}"

  tar -czf "$final_archive" -C "$report_dir" SUBJECT_FILE BODY_FILE ATTACHMENT_FILE.tar.gz

  # Переместите итоговый архив из временного каталога в каталог для отправки
  if [[  -f "$final_archive"  ]]; then
    mv "$final_archive" "$DIR_REPORTS_TO_SEND"
    if [[  $? -eq 0  ]]; then
      echo
      # func_echo_info "Перемещен файл: '$final_archive' в каталог отправки" "$DIR_REPORTS_TO_SEND" "$LOG_FILE_NAME_FULL"
    else
      func_echo_error "Не удалось преместить файл: '$final_archive' в каталог отправки" "$DIR_REPORTS_TO_SEND" "$LOG_FILE_NAME_FULL"
    fi
  else
    func_echo_error "Файл не найден" "$final_archive" "$LOG_FILE_NAME_FULL"
  fi

  func_echo_info "Всего отсканировано образов Docker" "$total_images" "$LOG_FILE_NAME_FULL"

  if [[  "$failed_scans" -gt 0  ]]; then
    func_echo_error "Не удалось отсканировать образов Docker" "$failed_scans" "$LOG_FILE_NAME_FULL"
  fi

  func_echo_info "Отчет" "${final_archive_name_only}" "$LOG_FILE_NAME_FULL"
  func_echo_info "Завершено (dockle) сканирование для пользователя" "$userUID" "$LOG_FILE_NAME_FULL"
  rm -rf "$temp_dir"
} # func_dockle_scan_docker_images_for_user()


# ФУНКЦИЯ Сканирования. Основная функция для работы с несколькими пользователями.
# Docker может быть запущен как от root, так и от любого другого пользователя в режиме rootless.
# Docker может быть запущен как при загрузке ОС, так и при входе пользователя в систему.
#   Существует возможность одновременного запуска нескольких демонов на одном хосте.
#     Эта функция признана разработчиками Docker экспериментальной.
#     При ее использовании пользователь может столкнуться с нерешенными проблемами.
#     А привычные в использовании возможности Docker могут работать неадекватно.
#     Данный режим работы признан не допустимым и должен ограничиваться административными методами.
# Поиск всех запущенных Docker и запуск сканеров
func_search_docker_sock_and_run_scanner() {
  if [ -z "$*" ]; then
    # нет аргументов
    func_echo_error_simple "Не указан вид сканирования" "$LOG_FILE_NAME_FULL"
    exit 1
  fi

  type_scan="$1"
  if [[ "$type_scan" != "trivy" && "$type_scan" != "bench" && "$type_scan" != "dockle" ]]; then
    func_echo_error "Неизвестный вид сканирования" "$type_scan" "$LOG_FILE_NAME_FULL"
    exit 1
  fi

  # Найдите все файлы сокетов Docker в системе
  local docker_sockets
  docker_sockets=$(find /run /var/run -type s -name "docker.sock" 2>/dev/null)

  if [[ -z "$docker_sockets" ]]; then
    func_echo_info_simple "В этой системе не найдено демонов Docker"  "$LOG_FILE_NAME_FULL"
    exit 0
  fi

  # Обработаем для каждого сокета
  while IFS= read -r docker_socket; do
    local owner_info
    owner_info=$(stat -c "%U %u" "$docker_socket")
    local username
    local uid
    username=$(echo "$owner_info" | awk '{print $1}')
    uid=$(echo "$owner_info" | awk '{print $2}')

    # Запуск сканирования
    case "$type_scan" in
      "trivy") func_trivy_scan_docker_images_for_user "$username" "$uid" "$docker_socket" ;;
      "dockle") func_dockle_scan_docker_images_for_user "$username" "$uid" "$docker_socket" ;;
      "bench") func_bench_scan_docker_user "$username" "$uid" "$docker_socket" ;;
      *) func_echo_error "Неизвестный вид сканирования" "$type_scan" "$LOG_FILE_NAME_FULL" ;;
    esac
  done <<<"$docker_sockets"
} # func_search_docker_sock_and_run_scanner


# сканирование образов утилитой trivy
func_scan_trivy() {
  echo
  # Убедитесь, что Trivy установлен
  func_check_packages "trivy" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  func_echo_info_simple "Сканирование trivy для всех доступных сокетов Docker начато" "$LOG_FILE_NAME_FULL"
  # Запустим сканирование
  func_search_docker_sock_and_run_scanner "trivy"
  func_echo_info_simple "Сканирование trivy для всех доступных сокетов Docker завершено" "$LOG_FILE_NAME_FULL"
  echo
} # func_scan_trivy

# сканирование утилитой docker-bench-security
func_scan_bench() {
  echo
  func_echo_info_simple "Сканирование docker-bench-security для всех доступных сокетов Docker начато" "$LOG_FILE_NAME_FULL"
  # Запустим сканирование
  func_search_docker_sock_and_run_scanner "bench"
  func_echo_info_simple "Сканирование docker-bench-security для всех доступных сокетов Docker завершено" "$LOG_FILE_NAME_FULL"
  echo
} # func_scan_bench

# сканирование образов утилитой dockle
func_scan_dockle() {
  echo
  # Убедитесь, что dockle установлен
  func_check_packages "dockle" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  func_echo_info_simple "Сканирование dockle для всех доступных сокетов Docker начато" "$LOG_FILE_NAME_FULL"
  # Запустим сканирование
  func_search_docker_sock_and_run_scanner "dockle"
  func_echo_info_simple "Сканирование dockle для всех доступных сокетов Docker завершено" "$LOG_FILE_NAME_FULL"
  echo
} # func_scan_dockle
