#!/usr/bin/bash

# eugene
# 2025.02.03

# Отправка почты


# ПРЕДВАРИТЕЛЬНАЯ ПРОВЕРКА

# Подключение сервисного файла
FILE_SCRIPTS_FUNCTIONS="/usr/sbin/docker_s_functions"

# Убедитесь, что файл существует
if [ ! -f "$FILE_SCRIPTS_FUNCTIONS" ]; then
  echo "$(basename "$0") Ошибка: Файл не найден: $FILE_SCRIPTS_FUNCTIONS."
  logger --id=$$ "$(basename "$0") Ошибка: Файл не найден: $FILE_SCRIPTS_FUNCTIONS."
  exit 1
fi

# Подключим source
source "$FILE_SCRIPTS_FUNCTIONS" || { \
  echo "$(basename "$0") Ошибка: Не подключен source файл: $FILE_SCRIPTS_FUNCTIONS."
  logger --id=$$ "$(basename "$0") Ошибка: Не подключен source файл: $FILE_SCRIPTS_FUNCTIONS."
  exit 1
}

# Имя файла лога
LOG_FILE_NAME="mail.log"

# Имя файла лога полное
LOG_FILE_NAME_FULL="$DIR_LOGS"/"$LOG_FILE_NAME"

# Проверка наличия файла настроек почты
func_check_file "$FILE_SEND_MAIL_SETTING" "$LOG_FILE_NAME_FULL"
if [ $? -ne 0 ]; then
  exit 1
fi

# ФУНКЦИИ СКРИПТА

# ФУНКЦИЯ отправки письма
func_send_email_from_archive() {
  local archive_file="$1"

  if [[ -z "$archive_file" ]]; then
    # нет аргумента
    func_echo_error_simple "Файл для отправки по электронной почте не предоставлен" "$LOG_FILE_NAME_FULL"
    return 1
  fi

  # Проверка наличия файла
  func_check_file "$archive_file" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    return 1
  fi

  # Создайте временный каталог со сложным случайным именем
  local temp_dir
  local temp_suffix="send_mail_temp_XXXXXXXX"
  temp_dir=$(mktemp -d -t ${temp_suffix})
  func_check_mkdir "${temp_dir}" "$LOG_FILE_NAME_FULL"
  if [ $? -ne 0 ]; then
    exit 1
  fi

  # Распакуйте архив во временный каталог
  tar -xzf "$archive_file" -C "$temp_dir"
  if [[ $? -ne 0 ]]; then
    func_echo_error "Не удалось распаковать файл архива" "$archive_file" "$LOG_FILE_NAME_FULL"
    rm -rf "$temp_dir"
    return 1
  fi

  # Определите пути к извлеченным файлам, которые будут затем отправляться
  # Заголовок письма
  local subject_file="$temp_dir/SUBJECT_FILE"
  # Тело письма
  local body_file="$temp_dir/BODY_FILE"
  # Прикрепленный файл (может отсутствовать)
  local attachment_file="$temp_dir/ATTACHMENT_FILE.tar.gz"

  # Проверьте, существуют ли необходимые файлы
  if [[ ! -f "$subject_file" || ! -f "$body_file"  ]]; then
    func_echo_error_simple "Файлы (SUBJECT_FILE, BODY_FILE) должны быть в архиве" "$LOG_FILE_NAME_FULL"
    rm -rf "$temp_dir"
    # Переместите файл для отправки в каталог забракованных файлов
    mv "$archive_file" "$DIR_REPORTS_BAD"
    if [[  $? -ne 0  ]]; then
      func_echo_error "Не удалось преместить файл: '$archive_file' в каталог отправки" "$DIR_REPORTS_BAD" "$LOG_FILE_NAME_FULL"
      return 1
    fi
    return 1
  fi

  # Проверьте наличие прикрепленного файла (вложения)
  if [[ ! -f "$attachment_file" ]]; then
    # echo "Инфо: Вложение (ATTACHMENT_FILE.tar.gz) не найдено. Пропустим его."
    # echo "$(date +"%Y-%m-%d_%H-%M-%S") Инфо: Вложение (ATTACHMENT_FILE.tar.gz) не найдено. Пропустим его." >> "$LOG_FILE_NAME_FULL"
    attachment_file="0"
  fi

  local settings_file="$FILE_SEND_MAIL_SETTING"

  # Загрузка настроек из конфигурационного файла
  # SMTP-сервер и учетные данные
  local smtp_server_type=$(jq -r '.smtp_server_type' "$settings_file")
  local smtp_server=$(jq -r '.smtp_server' "$settings_file")
  local smtp_port=$(jq -r '.smtp_port' "$settings_file")
  # Отправитель
  local sender=$(jq -r '.sender' "$settings_file")
  # Получатель
  local recipient=$(jq -r '.recipient' "$settings_file")
  # Получатель
  local recipient_copy=$(jq -r '.recipient_copy' "$settings_file")
  # Нужна ли аутентификация: 0 нет, 1 нужна
  local authentification=$(jq -r '.authentification' "$settings_file")

  # Проверьте, что все необходимые данные загружены
  if [[ -z "$smtp_server_type" || ("$smtp_server_type" == "null") || \
        -z "$smtp_server" || ("$smtp_server" == "null") || \
        -z "$smtp_port" || ("$smtp_port" == "null") || \
        -z "$authentification" || ("$authentification" == "null") || \
        -z "$recipient" || ("$recipient" == "null") ]]; then
    func_echo_error "Не все данные загружены из конфигурационного файла" "$settings_file" "$LOG_FILE_NAME_FULL"
    return 1
  fi

  if [[ -z "$recipient_copy" || ("$recipient_copy" == "null") ]]; then
    recipient_copy=""
  fi

  if [ "$authentification" == "1" ]; then
    # Учетные данные
    local smtp_user=$(jq -r '.smtp_user' "$settings_file")
    local smtp_pass=$(jq -r '.smtp_pass' "$settings_file")
  fi
  # Проверьте, что Учетные данные загружены
  if [[ -z "$smtp_user" || ("$smtp_user" == "null") || \
        -z "$smtp_pass" || ("$smtp_pass" == "null") ]]; then
    func_echo_error "Не загружены учетные данные из конфигурационного файла" "$settings_file" "$LOG_FILE_NAME_FULL"
    return 1
  fi

  # Заголовок
  local subject=$(<"$subject_file")

  # Тело письма
  local body=$(<"$body_file")

  # Строка границы (используется для разделения различных частей сообщения в формате MIME)
  boundary="----=_NextPart_$(date +%s)"

#${recipient_copy:+Cc: $recipient_copy}


  # Создайте содержимое электронной почты
  local email_content=$(cat <<EOF
From: $sender
To: $recipient

EOF
)
  if [ -n "$recipient_copy" ]; then
    email_content+=$(cat <<EOF
Cc: $recipient_copy

EOF
)
  fi

  email_content+=$(cat <<EOF

Subject: $subject
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="$boundary"

--$boundary
Content-Type: text/plain; charset="UTF-8"
Content-Transfer-Encoding: 7bit

$body

EOF
)

  # Допишите содержимое электронной почты
  if [ "$attachment_file" != "0" ]; then
    # вложение есть
    email_content+=$(cat <<EOF

--$boundary
Content-Type: application/octet-stream; name="$(basename "$attachment_file")"
Content-Transfer-Encoding: base64
Content-Disposition: attachment; filename="$(basename "$attachment_file")"

$(base64 "$attachment_file")

--$boundary--
EOF
)
  else
    # вложения нет
    email_content+=$(cat <<EOF

--$boundary--
EOF
)
  fi

  # Отправьте электронное письмо с помощью curl
  local response
  local response_status=0

  curl_cmd=(curl --url "$smtp_server_type://$smtp_server:$smtp_port" \
    --ssl-reqd \
    --mail-from "$smtp_user" \
    --mail-rcpt "$recipient")

  if [ -n "$recipient_copy" ]; then
    curl_cmd+=(--mail-rcpt "$recipient_copy")
  fi

  if [ "$authentification" == "1" ]; then
    curl_cmd+=(--user "$smtp_user:$smtp_pass")
  fi

  temp_email_file=$(mktemp)
  echo "$email_content" > "$temp_email_file"

  curl_cmd+=(-T "$temp_email_file")

  response=$("${curl_cmd[@]}" 2>&1)
#  response=$("${curl_cmd[@]}")
  response_status=$?

  rm "$temp_email_file"

  # Проверим статус отправки
  if [[ $response_status == 0 ]]; then
    # func_echo_info "Письмо отправлено для файла" "$archive_file" "$LOG_FILE_NAME_FULL"
    rm -rf "$temp_dir"

    # Переместите файл в отправленные
    mv "$archive_file" "$DIR_REPORTS_SEND"
    if [[  $? -ne 0  ]]; then
      func_echo_error "Не удалось преместить файл: '$archive_file' в каталог отправленные" "$DIR_REPORTS_SEND" "$LOG_FILE_NAME_FULL"
      rm -rf "$temp_dir"
      return 1
    fi
      rm -rf "$temp_dir"
      return 0
  else
    # func_echo_error "Письмо не отправлено для файла" "$archive_file" "$LOG_FILE_NAME_FULL"
    rm -rf "$temp_dir"
    return 1
  fi
}


# НАЧАЛО

if [ $# -gt 0 ]; then
  func_echo_info "Начата отправка письма для файла" "$1" "$LOG_FILE_NAME_FULL"

  func_send_email_from_archive "$1"
  status_send=$?

  if [[ $status_send == 0 ]]; then
    func_echo_info_simple "Письмо отправлено" "$LOG_FILE_NAME_FULL"
    # func_echo_info "Завершена отправка письма для файла" "$1" "$LOG_FILE_NAME_FULL"
  else
    func_echo_error_simple "Письмо не отправлено" "$LOG_FILE_NAME_FULL"
    # func_echo_error "Письмо не отправлено для файла" "$1" "$LOG_FILE_NAME_FULL"
  fi
else
  func_echo_error_simple "Для send_mail.sh не передано имя файла для отправки" "$LOG_FILE_NAME_FULL"
  exit 1
fi

