#!/usr/bin/bash

# eugene
# 2025-01-26

# Сервис docker_tracker.service запускает скрипт docker_s_tracker_monitor.sh.
# Скрипт docker_s_tracker_monitor.sh запускает скрипт запуска не отправленных по почте файлов docker_s_send_not_send_files.sh,
#   а затем запускает отслеживание каталога /var/log/docker_security/reports/to_send на наличие новых файлов
#   и запускает скрипт docker_s_tracker.sh при появлении файлов на отправку.
# Скрипт docker_s_tracker.sh получает на вход имена файлов и инициирует их отправку по почте.


# ПРЕДВАРИТЕЛЬНАЯ ПРОВЕРКА

# Подключение сервисного файла
FILE_SCRIPTS_FUNCTIONS="/usr/sbin/docker_s_functions"

# Убедитесь, что файл существует
if [ ! -f "$FILE_SCRIPTS_FUNCTIONS" ]; then
  echo "$(basename "$0") Ошибка: Файл не найден: $FILE_SCRIPTS_FUNCTIONS."
  logger --id=$$ "$(basename "$0") Ошибка: Файл не найден: $FILE_SCRIPTS_FUNCTIONS."
  exit 1
fi

# Подключим source
source "$FILE_SCRIPTS_FUNCTIONS" || { \
  echo "$(basename "$0") Ошибка: Не подключен source файл: $FILE_SCRIPTS_FUNCTIONS."
  logger --id=$$ "$(basename "$0") Ошибка: Не подключен source файл: $FILE_SCRIPTS_FUNCTIONS."
  exit 1
}

# Имя файла лога полное
LOG_FILE_TRACKER="$DIR_LOGS"/"tracker.log"

# Проверка наличия файла отправки почты
func_check_file "$FILE_SCRIPTS_SEND_MAIL" "$LOG_FILE_TRACKER"
if [ $? -ne 0 ]; then
  exit 1
fi


# НАЧАЛО

sleep 5

# Проверка наличия переданного параметра
if [[ -z "$1" ]]; then
  func_echo_error_simple "$(basename "$0") Файл для обработки не предоставлен" "$LOG_FILE_TRACKER"
  exit 1
fi

# Полное имя файла, который будет отправляться по почте
FILE_PATH="$1"

if [[ -f "$FILE_PATH" ]]; then
  # Обработка лог-файлов
  func_echo_info "Обработка файла" "$FILE_PATH" "$LOG_FILE_TRACKER"

  # Вызов отправки письма send_mail.sh
  if ! bash "$FILE_SCRIPTS_SEND_MAIL" "$FILE_PATH"; then
    func_echo_error "Сбой обработки файла" "$FILE_PATH" "$LOG_FILE_TRACKER"
  else
    func_echo_info "Завершена обработка файла" "$FILE_PATH" "$LOG_FILE_TRACKER"
  fi
else
  # Запись в логи, если файл не найден
  func_echo_error "Файл не найден" "$FILE_PATH" "$LOG_FILE_TRACKER"
  exit 1
fi

