#!/usr/bin/bash

# eugene
# 2025-01-26

# Сервис docker_tracker.service запускает скрипт docker_s_tracker_monitor.sh.
# Скрипт docker_s_tracker_monitor.sh запускает скрипт запуска не отправленных по почте файлов docker_s_send_not_send_files.sh,
#   а затем запускает отслеживание каталога /var/log/docker_security/reports/to_send на наличие новых файлов
#   и запускает скрипт docker_s_tracker.sh при появлении файлов на отправку.
# Скрипт docker_s_tracker.sh получает на вход имена файлов и инициирует их отправку по почте.


# ПРЕДВАРИТЕЛЬНАЯ ПРОВЕРКА

# Подключение сервисного файла
FILE_SCRIPTS_FUNCTIONS="/usr/sbin/docker_s_functions"

# Убедитесь, что файл существует
if [ ! -f "$FILE_SCRIPTS_FUNCTIONS" ]; then
  echo "$(basename "$0") Ошибка: Файл не найден: $FILE_SCRIPTS_FUNCTIONS."
  logger --id=$$ "$(basename "$0") Ошибка: Файл не найден: $FILE_SCRIPTS_FUNCTIONS."
  exit 1
fi

# Подключим source
source "$FILE_SCRIPTS_FUNCTIONS" || { \
  echo "Ошибка: Не подключен source файл: $FILE_SCRIPTS_FUNCTIONS."
  logger --id=$$ "$(basename "$0") Ошибка: Не подключен source файл: $FILE_SCRIPTS_FUNCTIONS."
  exit 1
}

# Имя файла лога полное
LOG_FILE_TRACKER="$DIR_LOGS"/"tracker.log"

# Проверка наличия файла отправки почты
func_check_file "$FILE_SCRIPTS_SEND_MAIL" "$LOG_FILE_TRACKER"
if [ $? -ne 0 ]; then
  exit 1
fi

# Проверка наличия файла отправки не отправленных файлов
func_check_file "$FILE_SCRIPTS_SEND_UNSENT" "$LOG_FILE_TRACKER"
if [ $? -ne 0 ]; then
  exit 1
fi

# Проверка наличия файла трекера
func_check_file "$FILE_SCRIPTS_TRACKER" "$LOG_FILE_TRACKER"
if [ $? -ne 0 ]; then
  exit 1
fi

# Убедитесь, что каталог на отправку по почте существует
func_check_mkdir "$DIR_REPORTS_TO_SEND" "$LOG_FILE_TRACKER"
if [ $? -ne 0 ]; then
  exit 1
fi

# Убедитесь, что inotify-tools установлен
func_check_packages "inotify-tools" "$LOG_FILE_TRACKER"
if [ $? -ne 0 ]; then
  exit 1
fi


# НАЧАЛО

# ЭТАП 1) Проверка наличия не отправленных файлов (т.е. пока слежение за каталогом было отключено, там могли складываться файлы)
func_echo_info_simple "Мониторинг по отправке писем запущен. Проверка наличия не отправленных файлов" "$LOG_FILE_TRACKER"

# ЗАПУСТИМ
"$FILE_SCRIPTS_SEND_UNSENT"

# ЭТАП 2) Запуск постоянного мониторинга за каталогом
# Запускается отслеживание на появление новых файлов в каталоге .../reports/to_send.
# Отслеживается, когда файл создается в целевом каталоге и когда файл перемещается из другого места в целевой каталог.
func_echo_info "Запуск постоянного мониторинга за каталогом" "$DIR_REPORTS_TO_SEND" "$LOG_FILE_TRACKER"

/usr/bin/inotifywait -m -e create -e moved_to --format '%w%f' "$DIR_REPORTS_TO_SEND" | xargs -I {} "$FILE_SCRIPTS_TRACKER" {}

