#!/usr/bin/bash

# eugene
# 2025-02-02

# Скрипт выполняет дополнительную предварительную настройку систему, выполняемую в ручном режиме
# в том случае, если программный комплекс был запущен уже после того, как некоторые пользователи были уже созданы

# Просмотрим все каталоги верхнего уровня в каталоге /home
#   1) daemon.json копируется из системного каталога в каталог каждого пользователя
#   2) для всех каталогов в /home в аудит вносится избранная информация в файл /etc/audit/rules.d/docker_users.rules
#   3) служба аудита перезапускается


# 1) Скопируем daemon.json из системного каталога в каталог каждого пользователя
WATCHED_DIR="/home"
cd "$WATCHED_DIR"
find * -maxdepth 0 -type d | while read -r NEW_ITEM; do
  DIR_DJ="$WATCHED_DIR""/""$NEW_ITEM""/.config/docker"
  mkdir -p "$DIR_DJ"
  # скопируем daemon.json в каталог пользователя
  cp /etc/docker_security/rootless-config/daemon.json "$DIR_DJ"
  FILE_DJ="$DIR_DJ""/""daemon.json"
  # установим права, запрещающие пользователю изменять этот файл
  chown root:root "$FILE_DJ"
  # сделаем файл защищенным
  chattr +i "$FILE_DJ"
done


# 2) для всех каталогов в /home в аудит вносится избранная информация в файл /etc/audit/rules.d/docker_users.rules
FILE_AUDIT="/etc/audit/rules.d/docker_users.rules"
cd "$WATCHED_DIR"
# аудит за всем каталогом ../.local/share/docker может генерировать большие объемы данных и вызывать быстрое заполнение дискового пространства
# в таком случае для аудита можно выбрать более простой режим контроля
# для этого необходимо закомментировать первую строку find... , следующую за этим комментарием, и расскомментировать следующую строку find...
find * -maxdepth 0 -type d | xargs -izDIRz echo -e '-w /home/zDIRz/.docker/ -k docker\n-w /home/zDIRz/.config/docker/ -k docker\n-w /home/zDIRz/.config/systemd/ -k docker\n-w /home/zDIRz/.local/share/docker/ -k docker' > "$FILE_AUDIT"
# find * -maxdepth 0 | xargs -izDIRz echo -e '-w /home/zDIRz/.docker/ -k docker\n-w /home/zDIRz/.config/docker/ -k docker\n-w /home/zDIRz/.config/systemd/ -k docker' > "$FILE_AUDIT"


# 3) служба аудита перезапускается
service auditd restart
if [ $? -ne 0 ]; then
  logger --id=$$ "$(basename "$0") Ошибка: при перезапуске сервиса аудита"
else
  logger --id=$$ "$(basename "$0") Инфо: сервис аудита перезапущен"
fi
